@@ -22,13 +22,14 @@ use std::{
     fmt::{Debug, Formatter},
     fs::{create_dir_all, read, read_dir, remove_dir_all, File},
     io::{BufRead, BufReader, Read},
+    iter,
     path::{Path, PathBuf},
     process::Command,
     time::Duration,
 };
 use subprocess::{CommunicateError, Exec, ExitStatus, NullFile, Redirection};
 
-const AUTO_SUFFIX: &str = "_fuzz::auto";
+const AUTO_GENERATED_SUFFIX: &str = "_fuzz::auto_generate";
 const ENTRY_SUFFIX: &str = "_fuzz::entry";
 
 const BASE_ENVS: &[(&str, &str)] = &[("TEST_FUZZ", "1"), ("TEST_FUZZ_WRITE", "0")];
@@ -57,6 +58,7 @@ enum SubCommand {
 }
 
 // smoelius: Wherever possible, try to reuse cargo test and libtest option names.
+#[remain::sorted]
 #[derive(Clap, Clone, Debug, Deserialize, Serialize)]
 #[clap(version = crate_version!())]
 struct TestFuzz {
@@ -90,12 +92,16 @@ struct TestFuzz {
     display_queue: bool,
     #[clap(long, about = "Target name is an exact name rather than a substring")]
     exact: bool,
-    #[clap(long, about = "Space or comma separated list of features to activate")]
-    features: Option<String>,
+    #[clap(
+        long,
+        multiple_occurrences = true,
+        about = "Space or comma separated list of features to activate"
+    )]
+    features: Vec<String>,
     #[clap(long, about = "List fuzz targets")]
     list: bool,
-    #[clap(long, about = "Resume target's last fuzzing session")]
-    resume: bool,
+    #[clap(long, value_name = "PATH", about = "Path to Cargo.toml")]
+    manifest_path: Option<String>,
     #[clap(long, about = "Do not activate the `default` feature")]
     no_default_features: bool,
     #[clap(
@@ -107,12 +113,12 @@ struct TestFuzz {
     no_run: bool,
     #[clap(long, about = "Disable user interface")]
     no_ui: bool,
+    #[clap(short, long, about = "Package containing fuzz target")]
+    package: Option<String>,
     #[clap(long, about = "Enable persistent mode fuzzing")]
     persistent: bool,
     #[clap(long, about = "Pretty-print debug output when displaying/replaying")]
     pretty_print: bool,
-    #[clap(short, long, about = "Package containing fuzz target")]
-    package: Option<String>,
     #[clap(
         long,
         about = "Replay corpus using uninstrumented fuzz target; to replay with instrumentation, \
@@ -135,18 +141,26 @@ struct TestFuzz {
     reset: bool,
     #[clap(long, hidden = true)]
     reset_all: bool,
+    #[clap(long, about = "Resume target's last fuzzing session")]
+    resume: bool,
     #[clap(long, about = "Stop fuzzing once a crash is found")]
     run_until_crash: bool,
     #[clap(long, about = "String that fuzz target's name must contain")]
     target: Option<String>,
+    #[clap(
+        long,
+        value_name = "NAME",
+        about = "Integration test containing fuzz target"
+    )]
+    test: Option<String>,
     #[clap(
         long,
         about = "Number of milliseconds to consider a hang when fuzzing or replaying (equivalent \
         to `-- -t <timeout>` when fuzzing)"
     )]
     timeout: Option<u64>,
-    #[clap(last = true, about = "Arguments for the fuzzer")]
-    args: Vec<String>,
+    #[clap(last = true, name = "args", about = "Arguments for the fuzzer")]
+    zargs: Vec<String>,
 }
 
 #[derive(Clone, Deserialize, Serialize)]
@@ -294,68 +308,98 @@ pub fn cargo_test_fuzz<T: AsRef<OsStr>>(args: &[T]) -> Result<()> {
 fn build(opts: &TestFuzz, quiet: bool) -> Result<Vec<Executable>> {
     let metadata = MetadataCommand::new().exec()?;
 
-    // smoelius: Put --message-format=json last so that it is easy to copy-and-paste the command
-    // without it.
     let mut args = vec![];
     if !opts.no_instrumentation {
         args.extend_from_slice(&["afl"]);
     }
-    args.extend_from_slice(&["test", "--frozen", "--no-run"]);
-    if let Some(features) = &opts.features {
-        args.extend_from_slice(&["--features", features]);
-    }
+    args.extend_from_slice(&["test", "--frozen", "--offline", "--no-run"]);
     if opts.no_default_features {
         args.extend_from_slice(&["--no-default-features"]);
     }
+    for features in &opts.features {
+        args.extend_from_slice(&["--features", features]);
+    }
     let target_dir = target_directory(true);
     let target_dir_str = target_dir.to_string_lossy();
     if !opts.no_instrumentation {
         args.extend_from_slice(&["--target-dir", &target_dir_str]);
     }
+    if let Some(path) = &opts.manifest_path {
+        args.extend_from_slice(&["--manifest-path", path]);
+    }
     if let Some(package) = &opts.package {
         args.extend_from_slice(&["--package", package]);
     }
     if opts.persistent {
-        args.extend_from_slice(&["--features", "test-fuzz/persistent"]);
+        args.extend_from_slice(&["--features", "test-fuzz/__persistent"]);
+    }
+    if let Some(name) = &opts.test {
+        args.extend_from_slice(&["--test", name]);
     }
-    args.extend_from_slice(&["--message-format=json"]);
 
     // smoelius: Suppress "Warning: AFL++ tools will need to set AFL_MAP_SIZE..." Setting
     // `AFL_QUIET=1` doesn't work here, so pipe standard error to /dev/null.
     // smoelius: Suppressing all of standard error is too extreme. For now, suppress only when
     // displaying/replaying.
-    let mut exec = Exec::cmd("cargo").args(&args).stdout(Redirection::Pipe);
+    let mut exec = Exec::cmd("cargo")
+        .args(
+            &args
+                .iter()
+                .chain(iter::once(&"--message-format=json"))
+                .collect::<Vec<_>>(),
+        )
+        .stdout(Redirection::Pipe);
     if quiet {
         exec = exec.stderr(NullFile);
     }
     debug!("{:?}", exec);
     let mut popen = exec.clone().popen()?;
-    let messages = popen
+    let artifacts = popen
         .stdout
         .take()
         .map_or(Ok(vec![]), |stream| -> Result<_> {
             let reader = BufReader::new(stream);
-            let messages: Vec<Message> = Message::parse_stream(reader)
+            let artifacts: Vec<Artifact> = Message::parse_stream(reader)
+                .filter_map(|result| match result {
+                    Ok(message) => match message {
+                        Message::CompilerArtifact(artifact) => Some(Ok(artifact)),
+                        _ => None,
+                    },
+                    Err(err) => Some(Err(err)),
+                })
                 .collect::<std::result::Result<_, std::io::Error>>()
                 .with_context(|| format!("`parse_stream` failed for `{:?}`", exec))?;
-            Ok(messages)
+            Ok(artifacts)
         })?;
     let status = popen
         .wait()
         .with_context(|| format!("`wait` failed for `{:?}`", popen))?;
 
-    ensure!(status.success(), "Command failed: {:?}", exec);
+    // smoelius: If the command failed, re-execute it without --message-format=json. This is easier
+    // than trying to capture and colorize `CompilerMessage`s like Cargo does.
+    if !status.success() {
+        let mut popen = Exec::cmd("cargo").args(&args).popen()?;
+        let status = popen
+            .wait()
+            .with_context(|| format!("`wait` failed for `{:?}`", popen))?;
+        ensure!(
+            !status.success(),
+            "Command succeeded unexpectedly: {:?}",
+            exec,
+        );
+        bail!("Command failed: {:?}", exec);
+    }
 
-    Ok(messages
+    Ok(artifacts
         .into_iter()
-        .map(|message| {
-            if let Message::CompilerArtifact(Artifact {
+        .map(|artifact| {
+            if let Artifact {
                 package_id,
                 target: build_target,
                 profile: ArtifactProfile { test: true, .. },
                 executable: Some(executable),
                 ..
-            }) = message
+            } = artifact
             {
                 let (test_fuzz_version, afl_version) =
                     test_fuzz_and_afl_versions(&metadata, &package_id)?;
@@ -934,7 +978,7 @@ fn fuzz(opts: &TestFuzz, executable: &Executable, target: &str) -> Result<()> {
             format!("{}", timeout * NANOS_PER_MILLI),
         ]);
     }
-    args.extend(opts.args.clone());
+    args.extend(opts.zargs.clone());
     args.extend(
         vec![
             "--",
@@ -952,8 +996,6 @@ fn fuzz(opts: &TestFuzz, executable: &Executable, target: &str) -> Result<()> {
         .status()
         .with_context(|| format!("Could not get status of `{:?}`", command))?;
 
-    // smoelius: When this command fails in a test, the stderr output gets lost. I haven't figured
-    // out why.
     ensure!(status.success(), "Command failed: {:?}", command);
 
     Ok(())
@@ -961,7 +1003,7 @@ fn fuzz(opts: &TestFuzz, executable: &Executable, target: &str) -> Result<()> {
 
 fn auto_generate_corpus(executable: &Executable, target: &str) -> Result<()> {
     let mut command = Command::new(&executable.path);
-    command.args(&["--exact", &(target.to_owned() + AUTO_SUFFIX)]);
+    command.args(&["--exact", &(target.to_owned() + AUTO_GENERATED_SUFFIX)]);
     debug!("{:?}", command);
     let status = command
         .status()
