@@ -16,7 +16,7 @@ fn display_qwerty() {
 fn display_debug_crash() {
     display(
         "debug",
-        "debug_crash::target_fuzz::auto",
+        "debug_crash::target_fuzz::auto_generate",
         "debug_crash::target",
         "",
         "Encountered a failure while not replaying. A buggy Debug implementation perhaps?",
@@ -27,7 +27,7 @@ fn display_debug_crash() {
 fn display_debug_hang() {
     display(
         "debug",
-        "debug_hang::target_fuzz::auto",
+        "debug_hang::target_fuzz::auto_generate",
         "debug_hang::target",
         "",
         "Encountered a timeout while not replaying. A buggy Debug implementation perhaps?",
@@ -37,7 +37,7 @@ fn display_debug_hang() {
 fn display(name: &str, test: &str, target: &str, stdout: &str, stderr: &str) {
     examples::test(name, test).unwrap().assert().success();
 
-    examples::test_fuzz(target)
+    examples::test_fuzz(name, target)
         .unwrap()
         .args(&["--display-corpus"])
         .assert()
